@@ -25,13 +25,9 @@ use 5.16.0;
 use strict;
 use warnings;
 
-require Exporter;
-use vars qw(@ISA @EXPORT);
-@ISA    = qw(Locale::Po4a::TransTractor);
-@EXPORT = qw();
+use parent qw(Locale::Po4a::TransTractor);
 
-use Locale::Po4a::TransTractor;
-use Locale::Po4a::Common;
+use Locale::Po4a::Common qw(wrap_mod dgettext);
 use YAML::Tiny;
 
 =head1 OPTIONS ACCEPTED BY THIS MODULE
@@ -111,6 +107,11 @@ equality of length of opening and closing block fences.
 
 Disable linting messages. When the source code cannot be fixed for clearer document structure, these messages are useless.
 
+=item B<cleanspaces>
+
+Remove extra spaces from the source segments in no-wrap mode. This is useful when the
+translation tools are sensitive to the number of spaces.
+
 =item B<yfm_keys>
 
 Comma-separated list of keys to process for translation in the YAML Front Matter
@@ -185,6 +186,11 @@ translatable attributes.
 If a minus sign (B<->) is prepended to the style name, then this
 attribute is not translated.
 
+If a percent signe (B<%>) is appended to the style name, then the
+style is processed as verbatim. The lines in paragraph or block of
+such style are preserved in the po files and the corresponding entries
+are marked as no-wrap.
+
 =item B<//po4a: entry >I<name>
 
 This declares an attribute entry as being translatable.  By default,
@@ -221,6 +227,7 @@ sub initialize {
     $self->{options}{'yfm_skip_array'} = 0;
     $self->{options}{'yfm_paths'}      = '';
     $self->{options}{'nolinting'}      = 0;
+    $self->{options}{'cleanspaces'}    = 0;
 
     foreach my $opt ( keys %options ) {
         die wrap_mod( "po4a::asciidoc", dgettext( "po4a", "Unknown option: %s" ), $opt )
@@ -256,7 +263,7 @@ sub initialize {
         entry => {}
     };
 
-    $self->register_attributelist('[verse,2,3,attribution,citetitle]');
+    $self->register_attributelist('[verse%,2,3,attribution,citetitle]');
     $self->register_attributelist('[quote,2,3,attribution,citetitle]');
     $self->register_attributelist('[icon]');
     $self->register_attributelist('[caption]');
@@ -290,7 +297,7 @@ sub register_attributelist {
     $list =~ s/\]$//;
     $list =~ s/\s+//;
     $list = "," . $list . ",";
-    $list =~ m/^,([-+]?)([^,]*)/;
+    $list =~ m/^,([-+]?)([^,%]*)/;
     my $command = $2;
     $self->{translate}->{$type}->{$command} = $list;
     print STDERR "Definition: $type $command: $list\n" if $debug{definitions};
@@ -355,14 +362,8 @@ sub parse_definition_file {
     close IN;
 }
 
-my $RE_SECTION_TEMPLATES = "sect1|sect2|sect3|sect4|preface|colophon|dedication|synopsis|index";
+
 my $RE_STYLE_ADMONITION  = "TIP|NOTE|IMPORTANT|WARNING|CAUTION";
-my $RE_STYLE_PARAGRAPH =
-  "normal|literal|verse|quote|listing|abstract|partintro|comment|example|sidebar|source|music|latex|graphviz";
-my $RE_STYLE_NUMBERING = "arabic|loweralpha|upperalpha|lowerroman|upperroman";
-my $RE_STYLE_LIST      = "appendix|horizontal|qanda|glossary|bibliography";
-my $RE_STYLES =
-  "$RE_SECTION_TEMPLATES|$RE_STYLE_ADMONITION|$RE_STYLE_PARAGRAPH|$RE_STYLE_NUMBERING|$RE_STYLE_LIST|float";
 
 BEGIN {
     my $UnicodeGCString_available = 0;
@@ -395,13 +396,20 @@ BEGIN {
 sub translate {
     my ( $self, $str, $ref, $type ) = @_;
     my (%options) = @_;
-    if ( ($options{'wrap'}==1) && ($str =~ / \+\n/) ) {
-        $options{'wrap'} = 0;
-        $str =~ s/([^+])\n/$1 /g;
-	$str =~ s/ \+\n/\n/g;
-        $str = $self->SUPER::translate( $str, $ref, $type, %options);
-	$str =~ s/\n/ +\n/g;
-	$options{'wrap'} = 1;
+    if ( $options{'wrap'} == 1 ) {
+        if ($str =~ / \+\n/) {
+            $options{'wrap'} = 0;
+            $str =~ s/([^+])\n/$1 /g;
+            $str =~ s/ \+\n/\n/g;
+            $str = $self->SUPER::translate( $str, $ref, $type, %options);
+            $str =~ s/\n/ +\n/g;
+            $options{'wrap'} = 1;
+        } else {
+            if ( $self->{options}{'cleanspaces'} == 1 ) {
+                $str =~ s/[ \n]+/ /g;
+            }
+            $str = $self->SUPER::translate( $str, $ref, $type, %options);
+        }
     } else {
         $str = $self->SUPER::translate( $str, $ref, $type, %options );
     }
@@ -544,6 +552,7 @@ sub parse {
             # Found title
 
             $wrapped_mode = 0;
+            undef $self->{type};
             my $level = $line;
             $level     =~ s/^(.).*$/$1/;
             $paragraph =~ s/\n$//s;
@@ -583,8 +592,8 @@ sub parse {
             do_paragraph( $self, $paragraph, $wrapped_mode );
             $wrapped_mode = 0;
             $paragraph    = "";
-	    $self->pushline( $titlelevel1 . $titlespaces);
-	    $title = $self->translate_indexterms($title);
+            $self->pushline( $titlelevel1 . $titlespaces);
+            $title = $self->translate_indexterms($title);
             my $t = $self->translate(
                 $title,
                 $self->{ref},
@@ -645,12 +654,12 @@ sub parse {
                     } elsif ( $t eq "_" ) {
 
                         # QuoteBlock
-                        if (    ( defined $self->{type} )
-                            and ( $self->{type} eq "verse" ) )
-                        {
+                        if ( ( defined $self->{type} )
+                             and (exists $self->{translate}->{style}->{$self->{type}})
+                             and $self->{translate}->{style}->{$self->{type}} =~ m/^,[^,%]+\%,/ ) {
                             $wrapped_mode = 0;
                             $self->{verbatim} = 1;
-                            print STDERR "QuoteBlock verse\n" if $debug{parse};
+                            print STDERR "QuoteBlock $self->{type}\n" if $debug{parse};
                         } else {
                             $wrapped_mode = 1;
                         }
@@ -690,19 +699,19 @@ sub parse {
         {
             # Found BlockId
             do_paragraph( $self, $paragraph, $wrapped_mode );
-	    my $block_id = $1;
-	    $paragraph    = "";
+            my $block_id = $1;
+            $paragraph    = "";
             $wrapped_mode = 1;
-	    if ($block_id =~ m/^([^,]+),(.+)$/) {
-		# Found BlockId with a xlabel
-		my $xlabel = $2;
-		$block_id = $1;
-		$self->pushline( "[[$block_id," );
-		do_paragraph( $self, $xlabel, 0);
-		$self->pushline( "]]\n" );
-	    } else {
-		$self->pushline( $line . "\n" );
-	    }
+            if ($block_id =~ m/^([^,]+),(.+)$/) {
+                # Found BlockId with a xlabel
+                my $xlabel = $2;
+                $block_id = $1;
+                $self->pushline( "[[$block_id," );
+                do_paragraph( $self, $xlabel, 0);
+                $self->pushline( "]]\n" );
+            } else {
+                $self->pushline( $line . "\n" );
+            }
             undef $self->{bullet};
             undef $self->{indent};
         } elsif ( not defined $self->{verbatim}
@@ -717,34 +726,17 @@ sub parse {
             $self->pushline($type);
             undef $self->{bullet};
             undef $self->{indent};
-        } elsif ( not defined $self->{verbatim}
-            and ( $line =~ m/^\[($RE_STYLES)\]$/ ) )
-        {
-            my $type = $1;
-            do_paragraph( $self, $paragraph, $wrapped_mode );
-            $paragraph    = "";
-            $wrapped_mode = 1;
-            $self->pushline( $line . "\n" );
-            if ( $type eq "verse" ) {
-                $wrapped_mode = 0;
-            }
-            undef $self->{bullet};
-            undef $self->{indent};
         } elsif ( not defined $self->{verbatim}
             and ( $line =~ m/^\[[^\]]+\]$/ ) )
         {
             do_paragraph( $self, $paragraph, $wrapped_mode );
             $paragraph = "";
-            my $t = $self->parse_style($line);
+            my ($type, $t) = $self->parse_style($line);
+            $self->{type} = $type;
             $self->pushline("$t\n");
             @comments     = ();
-            $wrapped_mode = 1;
-            if ( $line =~ m/^\[(['"]?)(verse|quote)\1,/ ) {
-                $self->{type} = $2;
-                if ( $self->{type} eq 'verse' ) {
-                    $wrapped_mode = 0;
-                }
-                print STDERR "Starting verse\n" if $debug{parse};
+            if ( exists $self->{translate}->{style}->{$type}  and $self->{translate}->{style}->{$type} =~ m/^,[^,%]+\%,/ ) {
+                $wrapped_mode = 0;
             }
             if ( ( ( $line =~ m/^\[format=(['"]?)(csv|tsv|dsv)\1,/ ) || ( $line =~ m/^\[separator=[^\|]/ ) )
                 && $self->{options}{'tablecells'} )
@@ -811,18 +803,18 @@ sub parse {
             my $attrname  = $1;
             my $attrsep   = $2;
             my $attrvalue = $3;
-	    my $linebreak = "";
+            my $linebreak = "";
             while ( $attrvalue =~ s/ ([\\+])$//s ) {
-		$linebreak = $1;
-		# add a carriage return at the end of attrvalue if there is none
-		$attrvalue .= "\n" if $attrvalue !~ m/\n$/;
+                $linebreak = $1;
+                # add a carriage return at the end of attrvalue if there is none
+                $attrvalue .= "\n" if $attrvalue !~ m/\n$/;
                 ( $line, $ref ) = $self->shiftline();
                 $ref  =~ m/^(.*):[0-9]+$/;
                 $line =~ s/^\s+|\s+$//;
-		print STDERR "appending attribute with $line" if $debug{parse};
+                print STDERR "appending attribute with $line" if $debug{parse};
                 $attrvalue .= $line;
             }
-	    print STDERR "attr definition: $attrvalue\n" if $debug{parse};
+            print STDERR "attr definition: $attrvalue\n" if $debug{parse};
             # Found an Attribute entry
             do_paragraph( $self, $paragraph, $wrapped_mode );
             $paragraph    = "";
@@ -837,10 +829,10 @@ sub parse {
                     "comment" => join( "\n", @comments ),
                     "wrap"    => 1
                 );
-		$t =~ s/\n/ \\\n/g;
+                $t =~ s/\n/ \\\n/g;
                 $self->pushline(":$attrname$attrsep$t\n");
             } else {
-		$attrvalue =~ s/\n/ $linebreak\n/g;
+                $attrvalue =~ s/\n/ $linebreak\n/g;
                 $self->pushline(":$attrname$attrsep$attrvalue\n");
             }
             @comments = ();
@@ -878,7 +870,7 @@ sub parse {
                 @comments = ();
             }
         } elsif ( not defined $self->{verbatim}
-		  and ($paragraph =~ m/^\s*$/)
+            and ( $paragraph =~ m/^\s*$/ )
             and ( $line !~ m/^\.\./ )
             and ( $line =~ m/^\.(\S.*)$/ ) )
         {
@@ -919,9 +911,9 @@ sub parse {
             $paragraph      = $text . "\n";
             $self->{indent} = "";
             $self->{bullet} = $bullet;
-	} elsif ( not defined $self->{verbatim}
-		  and ( $line eq " +") ) {
-	    $paragraph .= $line . "\n";
+        } elsif ( not defined $self->{verbatim}
+            and ( $line eq " +") ) {
+            $paragraph .= $line . "\n";
         } elsif ( ( $line =~ /^\s*$/ ) and ( !defined( $self->{type} ) or ( $self->{type} ne "Table" ) ) ) {
 
             # When not in table, empty lines or lines containing only spaces do break paragraphs
@@ -1061,19 +1053,19 @@ sub parse {
 
             if (   ( $paragraph ne "" && $self->{bullet} && length( $self->{indent} || "" ) == 0 ) )
             {
-		if ( ( !$self->{options}{'nolinting'} ) && ($paragraph !~ m/ \+\n/ ) ) {
-		    # Second line of an item block is not indented. It is unindented
-		    # (and allowed) additional text or a new list item.
-		    warn wrap_mod(
-			"$ref",
-			dgettext(
-			    "po4a",
-			    "It seems that you are adding unindented content to an item. "
-			    . "The standard allows this, but you may still want to change your document "
-			    . "to use indented text to provide better visual clues to writers."
-			)
-			);
-		}
+                if ( ( !$self->{options}{'nolinting'} ) && ($paragraph !~ m/ \+\n/ ) ) {
+                    # Second line of an item block is not indented. It is unindented
+                    # (and allowed) additional text or a new list item.
+                    warn wrap_mod(
+                        "$ref",
+                        dgettext(
+                            "po4a",
+                            "It seems that you are adding unindented content to an item. "
+                          . "The standard allows this, but you may still want to change your document "
+                          . "to use indented text to provide better visual clues to writers."
+                        )
+                    );
+                }
             } else {
                 undef $self->{bullet};
                 undef $self->{indent};
@@ -1167,13 +1159,13 @@ sub do_paragraph {
 
     my $end = "";
     if ($wrap) {
-        $paragraph =~ s/^(.*?)(\n*)$/$1/s;
-        $end = $2 || "";
+        $paragraph =~ s/(\n*)$//s;
+        $end = $1 || "";
     }
     if ( defined $self->{bullet} ) {
         my $bullet  = $self->{bullet};
         my $indent1 = $self->{indent};
-	$self->pushline($indent1 . $bullet);
+        $self->pushline($indent1 . $bullet);
     }
     $paragraph = $self->translate_indexterms($paragraph);
 
@@ -1189,7 +1181,7 @@ sub do_paragraph {
 
     my $unwrap_result = !$self->{options}{'forcewrap'} && $wrap && (! ($t =~ /\+\n/) ) ;
     if ($unwrap_result) {
-        $t =~ s/(\n| )+/ /g;
+        $t =~ s/[\n ]+/ /g;
     }
 
     @comments = ();
@@ -1214,35 +1206,35 @@ sub translate_in_regex {
     #  but that's consistant with the specification.
     my ($self, $paragraph, $pattern) = @_;
     if ( my @indexes = ($paragraph =~ m/$pattern/g ) ) {
-	for my $index (@indexes) {
-	    my @terms = ();
-	    while (
-		$index =~ m/\G(
-       "(?:[^"\\])+"               # quoted term
-       |  (?:[^,\\])+                # unquoted term
-         )(,\s*+)?/gx
-      )
-	    {
-		my $term = $1;
-		if ( $term =~ /^"(.*)"$/ ) {
-		    push @terms, '"' . ($self->translate(
-				  $1,
-				  $self->{ref},
-				  "Index entry",
-				  "wrap" => 1,
-				  "wrapcol" => 0)) . '"';
-		} else {
-		    push @terms, $self->translate(
-				  $term,
-				  $self->{ref},
-				  "Index entry",
-				  "wrap" => 1,
-				  "wrapcol" => 0);
-		}
-	    }
-	    $self->pushline("(((" . join (",", @terms) . ")))");
+        for my $index (@indexes) {
+            my @terms = ();
+            while (
+                $index =~ m/\G(
+                    "(?:[^"\\])+"               # quoted term
+                   | (?:[^,\\])+                # unquoted term
+                  )(,\s*+)?/gx
+              )
+            {
+                my $term = $1;
+                if ( $term =~ /^"(.*)"$/ ) {
+                    push @terms, '"' . ($self->translate(
+                            $1,
+                            $self->{ref},
+                            "Index entry",
+                            "wrap" => 1,
+                            "wrapcol" => 0)) . '"';
+                } else {
+                    push @terms, $self->translate(
+                        $term,
+                        $self->{ref},
+                        "Index entry",
+                        "wrap" => 1,
+                        "wrapcol" => 0);
+                }
+          }
+          $self->pushline("(((" . join (",", @terms) . ")))");
 
-	}
+        }
     }
     $paragraph =~ s/$pattern\n?//g;
     return $paragraph;
@@ -1264,7 +1256,9 @@ sub parse_style {
         return "[$t]";
     }
     my @attributes = $self->split_attributelist($text);
-    return "[" . join( ", ", $self->join_attributelist( "style", @attributes ) ) . "]";
+    my $type = $attributes[0];
+    $type =~ s/=.*//;
+    return ($type, "[" . join( ", ", $self->join_attributelist( "style", @attributes ) ) . "]");
 }
 
 sub parse_macro {
