@@ -393,22 +393,14 @@ under the terms of GPL v2.0 or later (see the COPYING file).
 =cut
 
 package Locale::Po4a::Man;
-use DynaLoader;
 
 use 5.16.0;
 use strict;
 use warnings;
 
-require Exporter;
-use vars qw(@ISA @EXPORT);
-@ISA    = qw(Locale::Po4a::TransTractor DynaLoader);
-@EXPORT = qw();                                        #  new initialize);
+use parent qw(Locale::Po4a::TransTractor);
 
-# Try to use a C extension if present.
-eval('bootstrap Locale::Po4a::Man "0.30"');
-
-use Locale::Po4a::TransTractor;
-use Locale::Po4a::Common;
+use Locale::Po4a::Common qw(wrap_mod wrap_ref_mod dgettext);
 
 use File::Spec;
 use Getopt::Std;
@@ -424,8 +416,8 @@ my $FONT_RE = "\\\\f(?:\\[[^\\]]*\\]|\\(..|[^\\(\\[])";
 
 # Variable used to identify non breaking spaces.
 # These non breaking spaces are used to ease the parsing, and a
-# translator can use them in her translation (and they will be translated
-# into the groff non-breaking space.
+# translator can use them in their translation (and they will be translated
+# into the groff non-breaking space).
 my $nbs;
 
 # Indicate if the page uses the mdoc macros
@@ -1023,7 +1015,7 @@ sub pre_trans {
         return $str;
     }
 
-    # Note: if you want to implement \c support, the gdb man page is your playground
+    # Note: if you want to implement \c support, the groff man page is your playground
     if ( not defined $self->{type} ) {
         $str =~ s/(\G|^(?:.*?)\n|^)        # Last position, or begin of a line
                    ([ \t]*[^.'][^\n]*(?<!\\)(?:\\\\)*) # the new line, which
@@ -1031,7 +1023,8 @@ sub pre_trans {
                    (?![ \t]*[.'])/$1$2/sgx;    # not followed by a command (.')
     }
     die wrap_ref_mod( $ref, "po4a::man",
-        dgettext( "po4a", "Escape sequence \\c encountered. This is not completely handled yet." ) )
+        dgettext( "po4a", "Escape sequence \\c encountered. This is not completely handled yet. Faulty input: %s" ),
+        $str )
       if ( $str =~ /\\c/ );
 
     $str =~ s/>/E<gt>/sg;
@@ -1095,6 +1088,7 @@ sub pre_trans {
     # non-breaking spaces
     # some non-breaking spaces may have been added during the parsing
     $str =~ s/\Q$nbs/\\ /sg;
+    $str =~ s/\\~/\\ /sg;
 
     print STDERR "$str\n" if ( $debug{'pretrans'} );
     return $str;
@@ -1680,25 +1674,25 @@ sub splitargs {
     # anymore. Let's play safe.
     $arguments =~ s/\\ /$nbs/g;
     $arguments =~ s/^ +//;
-    $arguments =~ s/\\&"/\\(dq/g;
     $arguments =~ s/^ *//;
     while ( length $arguments ) {
+        my $a;
         if ( $arguments =~ s/^"((?:[^"]|"")*)"(?!") *// ) {
-            my $a = $1;
+            $a = $1;
             $a =~ s/""/"/g if defined $a;
-            push @args, $a;
         } elsif ( $arguments =~ s/^"((?:[^"]|"")*) *$// ) {
 
             # Unterminated quote, but this seems to be handled by removing
             # the trailing spaces and closing the quotes.
-            my $a = $1;
+            $a = $1;
             $a =~ s/""/"/g if defined $a;
-            push @args, $a;
         } elsif ( $arguments =~ s/^([^ ]+) *// ) {
-            push @args, $1;
+            $a = $1;
         } else {
             die wrap_ref_mod( $ref, "po4a::man", dgettext( "po4a", "Cannot parse command arguments: %s" ), $arguments );
         }
+        $a =~ s/\\&"/\\(dq/g if (defined $a);
+        push @args, $a;
     }
     if ( $debug{'splitargs'} ) {
         print STDERR "ARGS=";
@@ -2034,6 +2028,13 @@ sub parse_tp_tq {
         ( $l2, $ref2 ) = $self->shiftline();
         chomp($l2);
     }
+
+    # Deal with \c line continuation in .TP or .TQ
+    while ( $l2 =~ s/\\c$// ) {
+        my ( $l3, $ref3 ) = $self->shiftline();
+        $l2 .= $l3;
+        chomp($l2);
+    }
     if ( $l2 =~ /^([.'][\t ]*([^\t ]*))(?:([\t ]+)(.*)$|$)/ ) {
         if ( $inline{$2} ) {
             my $tmp = "";
