@@ -42,8 +42,20 @@ Only the comments starting with 'TRANSLATORS' are added to the PO files to guide
 
 =head1 STATUS OF THIS MODULE
 
-This module is still beta.
-Please send feedback and feature requests.
+This module is still beta and not ready for production use.
+Please send patches to contribute, not bug reports as we don't know how to deal with them.
+
+=head1 OPTIONS ACCEPTED BY THIS MODULE
+
+These are this module's particular options:
+
+=over 4
+
+=item no-warn
+
+Do not warn about the current state of this module.
+
+=back
 
 =head1 SEE ALSO
 
@@ -70,14 +82,12 @@ use 5.16.0;
 use strict;
 use warnings;
 
-require Exporter;
-use vars qw($VERSION @ISA @EXPORT);
+use parent qw(Locale::Po4a::TeX);
+
+use vars qw($VERSION);
 $VERSION = $Locale::Po4a::TeX::VERSION;
-@ISA     = qw(Locale::Po4a::TeX);
-@EXPORT  = qw();
 
 use Locale::Po4a::Common;
-use Locale::Po4a::TeX;
 use subs qw(
   &parse_definition_file
   &register_generic_command &is_closed &translate_buffer
@@ -173,6 +183,10 @@ sub parse {
     my $t         = "";
     $docheader_pushed = 0;
 
+    print STDERR "The TexInfo module of po4a is not ready for production use, and needs a new maintainer.\n"
+      . "Please contact the po4a team if you want to help: send us patches, not bug reports.\n"
+      . "(use -o no-warn to remove this message)\n"
+      unless $self->{options}{'no-warn'};
   LINE:
     undef $self->{type};
     ( $line, $ref ) = $self->shiftline();
@@ -220,16 +234,16 @@ sub parse {
             $self->push_docheader();
             $self->pushline( $line . "\n" );
         } elsif ( $closed
-            and ( $line =~ /^@([^ ]*?)(?: +(.*))?$/ )
-            and ( defined $commands{$1} )
-            and ( $break_line{$1} ) )
+            and ( $line =~ /(^\s*)@([^ ]*?)(?: +(.*))?$/ )
+            and ( defined $commands{$2} )
+            and ( $break_line{$2} ) )
         {
+            my ($spaces, $cmd, $arg) = ($1, $2, $3);
             if ( length($paragraph) ) {
                 ( $t, @env ) = translate_buffer( $self, $paragraph, undef, @env );
                 $self->pushline($t);
                 $paragraph = "";
             }
-            my $arg  = $2;
             my @args = ();
             if ( defined $arg and length $arg ) {
 
@@ -237,8 +251,8 @@ sub parse {
                 $arg =~ s/\s*$//s;
                 @args = ( " ", $arg );
             }
-            ( $t, @env ) = &{ $commands{$1} }( $self, $1, "", \@args, \@env, 1 );
-            $self->pushline( $t . "\n" );
+            ( $t, @env ) = &{ $commands{$cmd} }( $self, $cmd, "", \@args, \@env, 1 );
+            $self->pushline( $spaces . $t . "\n" );
         } else {
 
             # continue the same paragraph
@@ -350,6 +364,7 @@ sub translate_buffer_menuentry {
         if ($description) {
             ( $t, @e ) = $self->translate_buffer( $description, $no_wrap, @env );
         }
+
         # Replace newlines with space for proper wrapping
         # See https://github.com/mquinson/po4a/issues/122
         $t =~ s/\n/ /sg;
@@ -381,15 +396,12 @@ sub translate_buffer_ignore {
 $translate_buffer_env{"ignore"} = \&translate_buffer_ignore;
 
 foreach (
-    qw(appendix section cindex findex kindex opindex pindex tindex vindex subsection
-    dircategory subtitle include
-    exdent center unnumberedsec
-    heading unnumbered unnumberedsubsec
-    unnumberedsubsubsec appendixsec appendixsubsec
-    appendixsubsubsec majorheading chapheading subheading
-    subsubheading shorttitlepage
-    subsubsection top item itemx chapter settitle
-    title author)
+    qw(appendix appendixsec appendixsubsec appendixsubsubsec author center
+    chapheading chapter cindex defline deftypeline dircategory exdent findex
+    heading include item itemx kindex majorheading opindex pindex section
+    settitle shorttitlepage subheading subsection subsubheading subsubsection
+    subtitle tindex title top unnumbered unnumberedsec unnumberedsubsec
+    unnumberedsubsubsec vindex)
   )
 {
     $commands{$_}               = \&line_command;
@@ -397,9 +409,9 @@ foreach (
     $translate_line_command{$_} = 1;
 }
 foreach (
-    qw(c comment clear set setfilename setchapternewpage vskip synindex
-    syncodeindex need fonttextsize printindex headings finalout sp
-    definfoenclose)
+    qw(c clear comment definfoenclose finalout fonttextsize frenchspacing
+    headings need printindex set setchapternewpage setfilename sp syncodeindex
+    synindex vskip)
   )
 {
     $commands{$_}   = \&line_command;
@@ -540,18 +552,17 @@ sub environment_line_command {
 #}
 #
 foreach (
-    qw(detailmenu menu titlepage group copying
-    documentdescription cartouche
-    direntry
-    ifdocbook ifhtml ifinfo ifplaintext iftex ifxml
-    ifnotdocbook ifnothtml ifnotinfo ifnotplaintext ifnottex ifnotxml)
-  )
+    qw(cartouche copying copyingg defblock detailmenu direntry displaymath
+    documentdescription group ifdocbook ifhtml ifinfo iflatex ifnotdocbook
+    ifnothtml ifnotinfo ifnotplaintextg ifnottex ifnotxml ifplaintext iftex
+    ifxml menu nodedescriptionblock titlepage)
+)
 {
     $commands{$_}               = \&environment_line_command;
     $translate_line_command{$_} = 0;
     $break_line{$_}             = 1;
 }
-foreach (qw(enumerate multitable ifclear ifset)) {
+foreach (qw(enumerate multitable ifclear ifset float linemacro)) {
     $commands{$_}   = \&environment_line_command;
     $break_line{$_} = 1;
 }
@@ -574,15 +585,11 @@ register_generic_command("*end,  ");
 $commands{'end'}   = $end_command;
 $break_line{'end'} = 1;
 
-register_generic_command("*macro,  ");
-$commands{'macro'}   = \&environment_command;
-$break_line{'macro'} = 1;
-register_generic_command("*itemize,  ");
-$commands{'itemize'}   = \&environment_command;
-$break_line{'itemize'} = 1;
-register_generic_command("*table,  ");
-$commands{'table'}   = \&environment_command;
-$break_line{'table'} = 1;
+foreach (qw(itemize macro ftable table vtable)) {
+    register_generic_command("*$_,  ");
+    $commands{$_}   = \&environment_command;
+    $break_line{$_} = 1;
+}
 
 # TODO: is_closed, use a regexp: \ does not escape the closing brace.
 # TBC on LaTeX.
