@@ -50,13 +50,9 @@ use 5.16.0;
 use strict;
 use warnings;
 
-require Exporter;
-use vars qw(@ISA @EXPORT);
-@ISA    = qw(Locale::Po4a::TransTractor);
-@EXPORT = qw();
+use parent qw(Locale::Po4a::TransTractor);
 
-use Locale::Po4a::TransTractor;
-use Locale::Po4a::Common;
+use Locale::Po4a::Common qw(wrap_mod dgettext);
 use YAML::Tiny;
 use Syntax::Keyword::Try;
 
@@ -784,11 +780,15 @@ sub parse_markdown {
         $paragraph = "";
         my ( $nextline, $nextref ) = $self->shiftline();
 
-        while ( $nextline !~ /^\s{0,3}$fence$fencechar*\s*$/ ) {
+        while ( $nextline && $nextline !~ /^\s{0,3}$fence$fencechar*\s*$/ ) {
             $paragraph .= "$nextline";
             ( $nextline, $nextref ) = $self->shiftline();
         }
-        do_paragraph( $self, $paragraph, $wrapped_mode, $type );
+        my $t = $self->translate(
+            $paragraph, $self->{ref}, $type,
+            "wrap"  => 0
+        );
+        $self->pushline($t);
         $self->pushline($nextline);
         $paragraph        = "";
         $end_of_paragraph = 1;
